////////////////////////////////////////////////////////////////////////////////////////
// File: scripts/injects/config.js
// Usage: injects config
////////////////////////////////////////////////////////////////////////////////////////

/**
 *
 * ENTITY INJECT SIGN:
 *
 * line0:   >> entity                         // required param
 * line1:   { inject_name }                   // required param
 * line2:   [offsetY]                         // optional | default: 0
 * line3:   [name]                            // optional | default: null
 *
*/

////////////////////////////////////////////////////////////////////////////////////////

/**
 * BLOCK INJECT SIGN (alpha):
 *
 * line0:   >> block                          // required param
 * line1:   { inject_name }                   // required param
 * line2:   <empty>
 * line3:   <empty>
 *
*/

////////////////////////////////////////////////////////////////////////////////////////

/**
 * @typedef {Object} Offset
 * @property {number} x - X-coordinate offset (default: 0.00).
 * @property {number} y - Y-coordinate offset (default: 0.00).
 * @property {number} z - Z-coordinate offset (default: 0.00).
 */

/**
 * @typedef {Object} EntityInject
 * @property {string} inject - The identifier for the inject (required).
 * @property {string} entity - The entity to be summoned (required).
 * @property {Offset} [offset] - Optional offset for the entity's spawn location.
 * @property {string} [spawnEvent] - Optional spawn event (default: "minecraft:entity_spawned").
 */

/**
 * @typedef {Object} BlockInject
 * @property {string} inject - The identifier for the inject (required).
 * @property {string} block - The block to be injected (required).
 * @property {null|Object} [permutation] - Optional permutation for the block (default: null).
 * @property {Offset} [offset] - Optional offset for the block's position (default: { x: 0, y: 0, z: 0 }).
 */

////////////////////////////////////////////////////////////////////////////////////////

export class EntityInjects {
  constructor() {
    /**
     * @type {EntityInject[]}
     */
    this.injects = [];
    this.initialize();
  }
  initialize() {
    this.injects = [
      {
        inject: "!custom_entity",                 // required param
        entity: "sbs_mw:custom_entity",           // required param
        offset: { x: 0.00, y: 0.00, z: 0.00 },    // optional | default: { x: 0.00, y: 0.00, z: 0.00 }
        spawnEvent: "minecraft:entity_spawned",   // optional | default: "minecraft:entity_spawned",
      },
      {
        inject: "!happy",
        entity: "sbs_mw:happy_billboard",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "minecraft:entity_spawned",
      },
      {
        inject: "!snow_wall",
        entity: "sbs_mw:snow_wall",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "minecraft:entity_spawned",
      },
      {
        inject: "!frozen_zombie",
        entity: "sbs_mw:frozen_zombie",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "minecraft:entity_spawned",
      },
      {
        inject: "!red_glowstick",
        entity: "sbs_mw:glow_stick",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:glowstick_red",
      },
      {
        inject: "!blue_glowstick",
        entity: "sbs_mw:glow_stick",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:glowstick_blue",
      },
      {
      inject: "!top_flag",
      entity: "sbs_mw:summit_flag",
      offset: { x: 0.00, y: 0.00, z: 0.00 },
      spawnEvent: "minecraft:entity_spawned",
      },
      {
        inject: "!s_camp",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: -1.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_campfire",
      },
      {
        inject: "!s_camp_1",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: -2.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_base_camp",
      },
      {
        inject: "!s_posing",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_posing_picture",
      },
      {
        inject: "!s_picture",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_taking_picture",
      },
      {
        inject: "!s_wind",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_pushed_by_wind",
      },
      {
        inject: "!s_ice_zombie",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: -2.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_breaking_frozen_mob",
      },
      {
        inject: "!s_poking",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_poking",
      },
      {
        inject: "!s_avalench",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_avalanche",
      },
      {
        inject: "!s_stretching",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_stretching",
      },
      {
        inject: "!s_checking_b",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_checking_backpack",
      },
      {
        inject: "!s_cleaning",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_camp_1_cleaning_trash",
      },
      {
        inject: "!s_digging_snow",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_digging_out_snow",
      },
      {
        inject: "!s_sleeping",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_camp1_sleeping",
      },
      {
        inject: "!s_sleeping1",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.85, y: 0.00, z: 0.20 },
        spawnEvent: "sbs_mw:sherpa_camp1_sleeping",
      },
      {
        inject: "!s_tired",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_tired_on_backpack",
      },
      {
        inject: "!checking_ladder",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_checking_ladder",
      },
      {
        inject: "!s_ladder_cl",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_climbing_ladder",
      },
      {
        inject: "!s_look_map",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_looking_at_map",
      },
      {
        inject: "!s_check_axe",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_checking_ice_axe",
      },
      {
        inject: "!camp_2_flag",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_camp_2_fixing_flag",
      },
      {
        inject: "!s_oxygen",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_oxygen",
      },
      {
        inject: "!s_scared",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa_lost_scared",
      },
      {
        inject: "!bucket_snowball",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:pile_of_snow",
      },
      {
        inject: "!sherpa23",
        entity: "sbs_mw:sherpa",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
        spawnEvent: "sbs_mw:sherpa23",
      },
      {
        inject: "!leaderboard",
        entity: "sbs_mw:leaderboard",
        offset: { x: 0.00, y: 0.00, z: 0.00 },
      },
      {
        inject: "#vanilla_entity",
        entity: "minecraft:spider",
      },
      {
        inject: "#spider",
        entity: "minecraft:spider",
      },
      {
        inject: "#zombie",
        entity: "minecraft:zombie",
      },
      {
        inject: "#zombie_adult",
        entity: "minecraft:zombie",
        spawnEvent: "minecraft:as_adult",
      },
      {
        inject: "#zombie_baby",
        entity: "minecraft:zombie",
        spawnEvent: "minecraft:as_baby",
      },
      {
        inject: "#npc",
        entity: "minecraft:npc",
      },
    ];
  }
  /**
   * @returns {EntityInject[]} The list of entity inject configurations.
   */
  getInjects() {
    return this.injects;
  }
}


////////////////////////////////////////////////////////////////////////////////////////


export class BlockInjects {
  constructor() {
    /**
     * @type {BlockInject[]}
     */
    this.injects = [];
    this.initialize();
  }
  initialize() {
    this.injects = [
      {
        inject: "!parkour_left",                  // required param
        block: "sbs_mw:parkour_edge_left",        // required param
        offset: { x: 0, y: 0, z: 0 },             // optional | default: { x: 0, y: 0, z: 0 }
      },
      {
        inject: "!parkour_right",
        block: "sbs_mw:parkour_edge_right",
      },
      {
        inject: "!ladder",
        block: "sbs_mw:ladder",
      },
      {
        inject: "!ladder_diagonal",
        block: "sbs_mw:ladder_diagonal",
      },
      {
        inject: "#light_block_15",
        block: "minecraft:light_block_15",
      },
    ];
  }
  /**
   * @returns {BlockInject[]} The list of block inject configurations.
   */
  getInjects() {
    return this.injects;
  }
}

////////////////////////////////////////////////////////////////////////////////////////