////////////////////////////////////////////////////////////////////////////////////////
// File: scripts/injects/injects.js
// Usage: handles world injects
////////////////////////////////////////////////////////////////////////////////////////

import { world, BlockComponentTypes } from "@minecraft/server";

import { removeInject, signToEntityRot, signToBlockRot, formatSignText } from "./utils.js";
import { EntityInjects, BlockInjects } from "./config.js";

////////////////////////////////////////////////////////////////////////////////////////

export function injectHandler(event) {
  const { dimension, location } = event.sourceBlock;

  // Obtain the block above the source block and extract relevant data
  const signBlock = dimension.getBlock(location).above();
  const { location: signLoc, permutation } = signBlock;
  const signRot = permutation.getState("ground_sign_direction");
  const signText = formatSignText(signBlock.getComponent(BlockComponentTypes.Sign)?.getText() || "");

  // Handle injection
  if (signText?.ln0) {  // check if inject is defined ::

    switch (signText.ln0) { // check if 1st line is defined

      // Handle entity injection
      case ">>entity":
        const entityRot = signToEntityRot(signRot); // Determine entity rotation based on sign rotation
        removeInject(dimension, signLoc, location); // clear cmd block and sign
        injectEntity(dimension, location, signText, entityRot); // inject entity
        break;

      // Handle block injection
      case ">>block":
        let blockRot = signToBlockRot(signRot) // Determine block rotation based on sign rotation
        if (blockRot != "invalid") {
          removeInject(dimension, signLoc, location); // clear cmd block and sign
          injectBlock(dimension, location, signText, blockRot); // inject block
        } else { // invalid sign rotation
          world.sendMessage(`§cInvalid inject sign rotation at §l${location.x} ${location.y} ${location.z}`);
        }
        break;

      default: // invalid 1st line ::
        world.sendMessage(`§cInvalid inject TYPE at §l${location.x} ${location.y} ${location.z}`);
        break;
    }

  } else { // no sign / text ::
    world.sendMessage(`§cUndefined inject at §l${location.x} ${location.y} ${location.z}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////

function injectEntity(dimension, location, signText, entityRot) {
  if (signText.ln1) { // check if 2nd line is defined
    let found = false;
    const entityInjectsInstance = new EntityInjects();
    const entityInjects = entityInjectsInstance.getInjects();

    for (const instance of entityInjects) { // iterate through injects config
      if (signText.ln1 === instance.inject) { // when match is found::
        dimension.runCommand(`/summon
          ${instance.entity}
          ${parseFloat(location.x) + (instance.offset?.x ?? 0)}
          ${parseFloat(location.y) + (instance.offset?.y ?? 0) + (parseFloat(signText.ln2) || 0)}
          ${parseFloat(location.z) + (instance.offset?.z ?? 0)}
          ${entityRot} 0
          ${instance.spawnEvent}
          "${signText.ln3}"
        `);
        found = true;
        return;
      }
    }
    // If no matching inject was found, send the invalid message
    if (!found) world.sendMessage(`§cInvalid inject NAME at §l${location.x} ${location.y} ${location.z}`);
  } else { // 2nd line not defined ::
    world.sendMessage(`§cUndefined inject NAME at §l${location.x} ${location.y} ${location.z}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////

// tbd...
function injectBlock(dimension, location, signText, blockRot) {
  if (signText.ln1) { // check if 2nd line is defined
    let found = false;
    const blockInjectsInstance = new BlockInjects();
    const blockInjects = blockInjectsInstance.getInjects();

    for (const instance of blockInjects) { // iterate through injects config
      if (signText.ln1 === instance.inject) { // when match is found::
        dimension.runCommand(`/setblock
          ${parseFloat(location.x) + (instance.offset?.x ?? 0)}
          ${parseFloat(location.y) + (instance.offset?.y ?? 0)}
          ${parseFloat(location.z) + (instance.offset?.z ?? 0)}
          ${instance.block} replace
        `);
        dimension.runCommand(`/setblock
          ${parseFloat(location.x) + (instance.offset?.x ?? 0)}
          ${parseFloat(location.y) + (instance.offset?.y ?? 0)}
          ${parseFloat(location.z) + (instance.offset?.z ?? 0)}
          ${instance.block} ["minecraft:cardinal_direction"="${blockRot}"] replace
        `);
        found = true;
        return;
      }
    }
    // If no matching inject was found, send the invalid message
    if (!found) world.sendMessage(`§cInvalid inject NAME at §l${location.x} ${location.y} ${location.z}`);
  } else { // 2nd line not defined ::
    world.sendMessage(`§cUndefined inject NAME at §l${location.x} ${location.y} ${location.z}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////