
// main imports
import { system } from "@minecraft/server";

// features
import * as util from "./util.js";
import { stunPlayer } from "../stun.js";

////////////////////////////////////////////////////////////////////////////////////////////////////

// list of durability reduction
const reduceDurability = {
  salmonSword: 5,
  piglinSword: 15,
  vexSword: 15,
  enderDragonSword: 20,
  caveSpiderSword: 15,
  creakingDagger: 10,
  vindicatorAxe: 5,
  witherScythe: 20,
  ironGolemHammer: 20,
  beeBlades: 15,
  polarBearClaw: 15,
  armadilloDrill: 20,
  zombieHand: 10,
  drownedTrident: 25,
  ravagerPunch: 15,
  breezeStaff: 15,
}

// animations
const abilityAnimation = {
  salmonSword: "animation.sbs_mw.salmon_sword.player.tp_ability",
  piglinSword: "animation.sbs_mw.piglin_sword.player.tp_ability",
  vexSword: "animation.sbs_mw.vex_sword.player.tp_ability",
  enderDragonSword: "animation.sbs_mw.ender_dragon_sword.player.tp_ability",
  caveSpiderSword: "animation.sbs_mw.cave_spider_sword.player.tp_ability",
  creakingDagger: "animation.sbs_mw.creaking_dagger.player.tp_ability",
  vindicatorAxe: "animation.sbs_mw.vindicator_axe.player.tp_ability",
  witherScythe: "animation.sbs_mw.wither_scythe.player.tp_ability",
  ironGolemHammer: "animation.sbs_mw.iron_golem_hammer.player.tp_ability",
  beeBlades: "animation.sbs_mw.bee_blades.player.tp_ability",
  polarBearClaw: "animation.sbs_mw.polar_bear_claw.player.tp_ability",
  armadilloDrill: "animation.sbs_mw.armadillo_drill.player.tp_ability",
  zombieHand: "animation.sbs_mw.zombie_hand.player.tp_ability",
  drownedTrident: "animation.sbs_mw.drowned_trident.player.tp_ability",
  ravagerPunch: "animation.sbs_mw.ravager_punch.player.tp_ability",
  breezeStaff: "animation.sbs_mw.breeze_staff.player.tp_ability",
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Special attack for melee weapons
////////////////////////////////////////////////////////////////////////////////////////////////////

export function armadilloDrill(event) {
  const player = event.source;
  const rotation = player.getRotation();
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.armadilloDrill} default 0 "v.is_first_person"`);
  player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:armadillo_drill_digger ^^0.5^2 ~~`);
  player.runCommand(`playsound sbs_mw.armadrillo_shoot @a[r=16] ~ ~ ~ 1 1`);
  util.damageItem(player, reduceDurability.armadilloDrill);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function beeBlades(event) {
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 35`);
  player.runCommand(`playsound sbs_mw.bee_blades_attack @a[r=16] ~ ~ ~ 1 1`);
  player.runCommand(`playanimation @s ${abilityAnimation.beeBlades} default 0 "v.is_first_person"`);
  if (target) {
    util.damageItem(player, reduceDurability.beeBlades);
    target.runCommand(`effect @s poison 4 1 false`);
    beeBladesAttack(player, target);
  }
}

function beeBladesAttack(player, target) {
  let attackCount = 0;
  const maxAttacks = 2;
  const delay = 10; // delay in ticks
  const dealDamage = () => {
    if (attackCount < maxAttacks) {
      target.applyDamage(5, { cause: "entityAttack", damagingEntity: player });
      attackCount++;
      system.runTimeout(dealDamage, delay); // Schedule the next attack
    }
  };
  dealDamage(); // Start the first attack
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function caveSpiderSword(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.caveSpiderSword} default 0 "v.is_first_person"`);
  util.shootMeleeProjectile("sbs_mw:cave_spider_sword_projectile", player,1.25);
  player.runCommand(`playsound sbs_mw.spider.shoot @a[r=16] ~ ~ ~ 1 1.2`);
  util.damageItem(player, reduceDurability.caveSpiderSword);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function creakingDagger(event) {
  const abilityDelay = 4; // delay in ticks
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`playanimation @s ${abilityAnimation.creakingDagger} default 0 "v.is_first_person"`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 32`);
  player.runCommand(`playsound sbs_mw.creaking_attack @a[r=16] ~ ~ ~ 1 1`);
  target ? target.runCommand(`summon sbs_mw:creaking_dagger_util ~ ~ ~`)
         : player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:creaking_dagger_util ^^^1 facing ^^^4`);
  const ability = () => {
    const target = util.getTargetEntity(player);
    if (target) {
      util.damageItem(player, reduceDurability.creakingDagger);
      creakingAttack(player, target);
    }
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

function creakingAttack(player, target) {
  let attackCount = 0;
  const maxAttacks = 2;
  const delay = 10; // delay in ticks
  const dealDamage = () => {
    if (attackCount < maxAttacks) {
      target.applyDamage(4, { cause: "entityAttack", damagingEntity: player });
      attackCount++;
      system.runTimeout(dealDamage, delay); // Schedule the next attack
    }
  };
  dealDamage(); // Start the first attack
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function drownedTrident(event) {
  const abilityDelay = 8; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 15`);
  player.runCommand(`playanimation @s ${abilityAnimation.drownedTrident} default 0 "v.is_first_person"`);
  const ability = () => {
    util.shootMeleeProjectile("sbs_mw:drowned_trident_projectile", player,1.25);
    util.damageItem(player, reduceDurability.drownedTrident);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function enderDragonSword(event) {
  const abilityDelay = 15; // delay in ticks
  const player = event.source;
  player.runCommand(`playanimation @s ${abilityAnimation.enderDragonSword} default 0 "v.is_first_person"`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 36`);
  player.runCommand(`playsound sbs_mw.ender_sword_cast @a[r=16] ~ ~ ~ 1 1`);
  player.runCommand(`particle sbs_mw:ender_dragon_sword_cast ~ ~ ~`);
  const ability = () => {
    player.dimension.spawnEntity("sbs_mw:ender_dragon_sword_breath", player.location);
    util.damageItem(player, reduceDurability.enderDragonSword);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function ironGolemHammer(event) {
  const abilityDelay = 3; // delay in ticks
  const player = event.source;
  const targets = util.getEntitiesFromRadius(player, 4)
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 16`);
  player.runCommand(`playanimation @s ${abilityAnimation.ironGolemHammer} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw:iron_golem_hammer_smash @a[r=16] ~ ~ ~ 1 1`);
  const ability = () => {
    player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:iron_golem_hammer_particle ^^^2 facing ^^^4`);
    try {
      if (targets) {
        for (const target of targets) {
          target.applyDamage(4, { cause: "entityAttack", damagingEntity: player })
          ironGolemHammerKnockback(player, target);
        }
        util.damageItem(player, reduceDurability.ironGolemHammer);
      }
    } catch {
      console.log(`ERROR: Player ${source.name} could not apply knockback to ${targets.typeId}`);
    }
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability

}

function ironGolemHammerKnockback(source, target) {
  try {
    const dir = source.getViewDirection();
    target.applyKnockback(dir.x, dir.z, util.rand(0.0, 0.3), util.rand(1.8, 3.5));
  } catch (err) {
    console.log(`KNOCKBACK ERROR :: ${err}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function piglinSword(event) {
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.piglinSword} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.piglin_sword_ability @a[r=16] ~ ~ ~ 1.0 1.0`);
  player.runCommand(`playsound sbs_mw.piglin_sword_ability2 @a[r=16] ~ ~ ~ 1.0 1.0`);
  if (target) {
    if (target.typeId == "minecraft:player") {
      target.applyDamage(4, { cause: "entityAttack", damagingEntity: player })
      player.runCommand(`particle sbs_mw:piglin_sword_hit ${target.location.x} ${target.location.y} ${target.location.z}`);
      stunPlayer(target, 2)
    } else {
      player.runCommand(`particle sbs_mw:piglin_sword_hit ${target.location.x} ${target.location.y} ${target.location.z}`);
      target.dimension.setBlockType(target.location, "minecraft:gold_block")
      player.runCommand(`playsound sbs_mw.piglin_sword_transform_block @a[r=16] ~ ~ ~ 1.0 1.0`);
      target.remove();
    }
    util.damageItem(player, reduceDurability.piglinSword);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function polarBearClaw(event) {
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.polarBearClaw} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.polar_bear_claw_ability @a[r=16] ~ ~ ~ 1.0 1.0`);
  target ? target.runCommand(`summon sbs_mw:polar_bear_util ~ ~ ~`)
         : player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:polar_bear_util ^^^1 facing ^^^4`);
  if (target) {
    target.applyDamage(15, { cause: "entityAttack", damagingEntity: player })
    polarBearClawKnockback(player, target);
    util.damageItem(player, reduceDurability.polarBearClaw);
  }
}

function polarBearClawKnockback(player, target) {
  const rot = player.getRotation();
  const yaw = rot.y; // Player's yaw in degrees
  const yawRad = yaw * (Math.PI / 180); // Convert yaw to radians

  const horizontalStrength = 1.8; // Strength of the knockback
  const angleOffset = Math.PI / 4; // 45 degrees offset for leftward knockback
  const knockbackAngle = yawRad - angleOffset; // Adjusted knockback angle

  // Calculate knockback directions
  const directionX = Math.cos(knockbackAngle) * horizontalStrength; // Forward knockback
  const directionZ = Math.sin(knockbackAngle) * horizontalStrength; // Leftward knockback

  // Apply camera shake for the target
  target.runCommand(`camerashake add @s 2.0 0.5 positional`);

  // Apply knockback to the target
  try {
    target.applyKnockback(directionX, directionZ, horizontalStrength, 0.5);
  } catch (err) {
    console.log(`KNOCKBACK ERROR :: ${err}`);
  }
}


////////////////////////////////////////////////////////////////////////////////////////////////////

export function ravagerPunch(event) {
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 32`);
  player.runCommand(`playanimation @s ${abilityAnimation.ravagerPunch} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.ravager_punch_hit @a[r=16] ~ ~ ~ 0.6 1.3`);
  if (target) {
    target.runCommand(`particle sbs_mw:ravager_punch_hit ~ ~ ~`);
    target.applyDamage(10, { cause: "entityAttack", damagingEntity: player })
    ravagerPunchKnockback(player, target);
    util.damageItem(player, reduceDurability.ravagerPunch);
  }
}

function ravagerPunchKnockback(source, target) {
  try {
    const dir = source.getViewDirection();
    target.applyKnockback(dir.x, dir.z, 7.5, 1.3);
  } catch {
    console.log(`ERROR: Player ${source.name} could not apply knockback to ${target.typeId}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function salmonSword(event) {
  const player = event.source;
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 30`);
  player.runCommand(`playanimation @s ${abilityAnimation.salmonSword} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.salmon_ability @a[r=16] ~ ~ ~ 1.4 0.6`);
  if (target) {
    player.runCommand(`particle sbs_mw:salmon_sword_hit ${target.location.x} ${target.location.y} ${target.location.z}`);
    target.applyDamage(3, { cause: "entityAttack", damagingEntity: player })
    salmonSwordKnockback(player, target);
    util.damageItem(player, reduceDurability.salmonSword);
  }
}

function salmonSwordKnockback(source, target) {
  try {
    const dir = source.getViewDirection();
    target.applyKnockback(dir.x, dir.z, 3.5, 0.85);
  } catch {
    console.log(`ERROR: Player ${source.name} could not apply knockback to ${target.typeId}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function vexSword(event) {
  const abilityDelay = 8; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.vexSword} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.vex.charge @a[r=16] ~ ~ ~ 1 1.5`);
  const ability = () => {
    player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:vex_sword_mob ^^^2 ~~`)
    util.damageItem(player, reduceDurability.vexSword);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function vindicatorAxe(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 16`);
  player.runCommand(`playanimation @s ${abilityAnimation.vindicatorAxe} default 0 "v.is_first_person"`);
  util.shootMeleeProjectile("sbs_mw:vindicator_axe_projectile", player)
  player.runCommand(`playsound sbs_mw.vindicator_axe_ability @a[r=16] ~ ~ ~ 1.0 1.0`);
  util.damageItem(player, reduceDurability.vindicatorAxe);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function witherScythe(event) {
  const player = event.source;
  const targets = util.getEntitiesFromCone(player, 3, 60);
  const target = util.getTargetEntity(player);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 13`);
  player.runCommand(`playanimation @s ${abilityAnimation.witherScythe} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.wither_scythe_ability @a[r=16] ~ ~ ~ 1.0 1.0`);
  player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:wither_scythe_util ^^^1 facing ^^^4`);
  if (targets) {
    for (const target of targets) {
      // stun
      if (target.typeId == "minecraft:player") {
        stunPlayer(target, 2)
      } else target.runCommand(`effect @s slowness 2 255 false`);
      // damage
      target.applyDamage(5, { cause: "entityAttack", damagingEntity: player })
      // sound
      player.runCommand(`playsound sbs_mw.wither_scythe_sword_hit_sound @a[r=16] ~ ~ ~ 1.5 1.0`);
    }
    util.damageItem(player, reduceDurability.witherScythe);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function zombieHand(event) {
  const abilityDelay = 8; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.zombieHand} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.zombie_hand_swing @a[r=16] ~ ~ ~`);
  const ability = () => {
    const target = util.getTargetEntity(player);
    try {
      if (target) {
        target.applyDamage(7, { cause: "entityAttack", damagingEntity: player })
        target.runCommand(`effect @s hunger 3 1 false`);
        zombieHandKnockback(player, target);
        util.damageItem(player, reduceDurability.zombieHand);
        player.runCommand(`playsound sbs_mw.zombie_hand_slap @a[r=16] ~ ~ ~`);
      }
    } catch (error) {
      console.log(`ERROR: Player ${source.name} could not apply knockback to ${target.typeId}`);
    }

  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

function zombieHandKnockback(source, target) {
  try {
    const dir = source.getViewDirection();
    target.applyKnockback(dir.x, dir.z, 3.8, 1.1);
  } catch {
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function breezeStaff(event) {
  const abilityDelay = 8; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.breezeStaff} default 0 "v.is_first_person"`);
  const ability = () => {
    const rotation = player.getRotation();
    player.runCommand(`execute rotated as @s rotated ~ 0 run summon sbs_mw:breeze_staff_tornado ^^0.5^2 ~~`);
    util.damageItem(player, reduceDurability.breezeStaff);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

export function tornadoMovement(event) {
  const floatMessage = parseFloat(event.message);
  const viewDirection = event.sourceEntity.getViewDirection();
  const directionX = viewDirection.x;
  const directionZ = viewDirection.z;
  try {
    event.sourceEntity.applyKnockback(directionX, directionZ, floatMessage, 0);
  } catch (err) {
    console.log(`KNOCKBACK ERROR :: ${err}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
