
// main imports
import { system } from "@minecraft/server";

// features
import * as util from "./util.js";

////////////////////////////////////////////////////////////////////////////////////////////////////

// list of durability reduction
const reduceDurability = {
  witchWand: 10,
  creeperCannon: 15,
  endermanPhaser: 10,
  blazeFlamethrower: 30,
  ghastBlaster: 30,
  spiderNet: 15,
  llamaSpitter: 15,
  magmaFlooder: 15,
  wardenSonicBlaster: 5,
  skeletonBow: 5,
  boggedBow: 20,
  pillagerCrossbow: 15,
  strayBow: 15,
}

// animations
const abilityAnimation = {
  witchWand: "animation.sbs_mw.witch_wand.player.tp_ability",
  creeperCannon: "animation.sbs_mw.creeper_cannon.player.tp_ability",
  endermanPhaser: "animation.sbs_mw.enderman_phaser.player.tp_ability",
  blazeFlamethrower: "animation.sbs_mw.blaze_flamethrower.player.tp_ability",
  ghastBlaster: "animation.sbs_mw.ghast_blaster.player.tp_ability",
  spiderNet: "animation.sbs_mw.spider_net.player.tp_ability",
  llamaSpitter: "animation.sbs_mw.llama_spitter.player.tp_ability",
  magmaFlooder: "animation.sbs_mw.magma_flooder.player.tp_ability",
  wardenSonicBlaster: "animation.sbs_mw.warden_sonic_blaster.player.tp_ability",
  skeletonBow: "animation.sbs_mw.skeleton_bow.player.tp_ability",
  boggedBow: "animation.sbs_mw.bogged_bow.player.tp_ability",
  pillagerCrossbow: "animation.sbs_mw.pillager_crossbow.player.tp_ability",
  strayBow: "animation.sbs_mw.stray_bow.player.tp_ability",
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Special attack for ranged weapons
////////////////////////////////////////////////////////////////////////////////////////////////////

export function boggedBow(event) {
  const abilityDelay = 2; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.boggedBow} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.bogged_bow_shoot @a[r=16] ~ ~ ~ 1 1`);
  const ability = () => {
    util.shootRangedProjectile(4, "sbs_mw:bogged_bow_projectile", player,1.25);
    util.damageItem(player, reduceDurability.boggedBow);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function witchWand(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.witchWand} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:witch_wand_projectile", player,1.25);
  player.runCommand(`playsound sbs_mw.witch_wand_shoot @a[r=16] ~ ~ ~ 1 1`);
  util.damageItem(player, reduceDurability.witchWand);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function creeperCannon(event) {
  const abilityDelay = 2; // delay in ticks
  const player = event.source;
  const ability = () => {
    player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
    player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 15`);
    player.runCommand(`playanimation @s ${abilityAnimation.creeperCannon} default 0 "v.is_first_person"`);
    util.shootRangedProjectile(4, "sbs_mw:creeper_cannon_projectile", player,1.25);
    player.runCommand(`playsound sbs_mw.creeper_shoot @a[r=16] ~ ~ ~ 1 1.2`);
    util.damageItem(player, reduceDurability.creeperCannon);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function endermanPhaser(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 16`);
  player.runCommand(`playanimation @s ${abilityAnimation.endermanPhaser} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:enderman_phaser_projectile", player,1.35);
  player.runCommand(`playsound sbs_mw.enderman_phaser_ability @a[r=16] ~ ~ ~ 1 1`);
  util.damageItem(player, reduceDurability.endermanPhaser);
}

export function randomTeleport(event) {
  const source = event.sourceEntity;
  const r = parseInt(event.message) || 10; // Default radius 10 if not provided
  const dimension = source.dimension;
  const startPos = source.location;

  // Generate all possible positions within the cube radius
  let positions = [];
  for (let dx = -r; dx <= r; dx++) {
    for (let dz = -r; dz <= r; dz++) {
      for (let dy = -r; dy <= r; dy++) {
        positions.push({ x: startPos.x + dx, y: startPos.y + dy, z: startPos.z + dz });
      }
    }
  }
  // Shuffle positions randomly
  shuffleArray(positions);

  // Check positions one by one in random order
  for (const pos of positions) {
    const y = findValidY(dimension, pos.x, pos.y, pos.z);
    if (y !== null) {
      source.teleport({ x: pos.x, y, z: pos.z }, { dimension });
      return;
    }
  }

}

// Finds a valid Y-coordinate for teleportation
function findValidY(dimension, x, startY, z) {
  for (let y = Math.max(startY - 10, -64); y < Math.min(startY + 10, 319); y++) {
    const blockBelow = dimension.getBlock({ x: x, y: y - 1, z: z });
    const blockAt = dimension.getBlock({ x: x, y: y, z });
    const blockAbove = dimension.getBlock({ x: x, y: y + 1, z: z });

    if (
      blockBelow && blockAt && blockAbove && // Ensure blocks are loaded
      blockAt.isAir && // Target block must be air
      blockAbove.isAir && // Block above must be air
      !blockBelow.isAir && !blockBelow.isLiquid // Block below must be solid
    ) {
      return y;
    }
  }
  return null;
}

// Shuffles an array in-place using Fisher-Yates algorithm
function shuffleArray(array) {
  for (let i = array.length - 1; i > 0; i--) {
    const j = Math.floor(Math.random() * (i + 1));
    [array[i], array[j]] = [array[j], array[i]];
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////

export function blazeFlamethrower(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.blazeFlamethrower} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.flamethrower_shoot @a[r=16] ~ ~ ~ 1 1`);
  let attackCount = 0;
  const maxAttacks = 10;
  const delay = 6; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(4, "sbs_mw:blaze_flamethrower_projectile", player,1.28);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  util.damageItem(player, reduceDurability.blazeFlamethrower);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function ghastBlaster(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 7`);
  player.runCommand(`playanimation @s ${abilityAnimation.ghastBlaster} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:ghast_blaster_projectile", player,1.35);
  player.runCommand(`playsound sbs_mw.ghast_blaster_ability @a[r=16] ~ ~ ~ 0.5 1.5`);
  util.damageItem(player, reduceDurability.ghastBlaster);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function spiderNet(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 10`);
  player.runCommand(`playanimation @s ${abilityAnimation.spiderNet} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:spider_net_projectile", player,1.25);
  player.runCommand(`playsound sbs_mw.spider.shoot @a[r=16] ~ ~ ~ 1 1.2`);
  util.damageItem(player, reduceDurability.spiderNet);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function llamaSpitter(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 14`);
  player.runCommand(`playanimation @s ${abilityAnimation.llamaSpitter} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.llama_spitter_spit @a[r=16] ~ ~ ~ 1.0 1.2`);
  let attackCount = 0;
  const maxAttacks = 5;
  const delay = 2; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(3, "sbs_mw:llama_spitter_projectile", player,1.25);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  util.damageItem(player, reduceDurability.llamaSpitter);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function magmaFlooder(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.magmaFlooder} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:magma_flodder_projectile", player,1.25);
  player.runCommand(`playsound sbs_mw.magma_flooder_ability @a[r=16] ~ ~ ~ 0.5 0.9`);
  util.damageItem(player, reduceDurability.magmaFlooder);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function wardenSonicBlaster1(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 20`);
  player.runCommand(`playanimation @s ${abilityAnimation.wardenSonicBlaster} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(2, "sbs_mw:warden_sonic_blaster_projectile1", player,1.25);
  player.runCommand(`playsound sbs_mw.warden.sonic_shoot @a[r=16] ~ ~ ~ 0.8 1.3`);
  util.damageItem(player, reduceDurability.wardenSonicBlaster);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function wardenSonicBlaster2(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 40`);
  player.runCommand(`playanimation @s ${abilityAnimation.wardenSonicBlaster} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(3, "sbs_mw:warden_sonic_blaster_projectile2", player,1.25);
  player.runCommand(`playsound sbs_mw.warden.sonic_shoot @a[r=16] ~ ~ ~ 0.8 1.3`);
  util.damageItem(player, reduceDurability.wardenSonicBlaster);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function wardenSonicBlaster3(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 60`);
  player.runCommand(`playanimation @s ${abilityAnimation.wardenSonicBlaster} default 0 "v.is_first_person"`);
  util.shootRangedProjectile(4, "sbs_mw:warden_sonic_blaster_projectile3", player,1.25);
  player.runCommand(`playsound sbs_mw.warden.sonic_shoot @a[r=16] ~ ~ ~ 0.8 1.3`);
  util.damageItem(player, reduceDurability.wardenSonicBlaster);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function skeletonBow(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.skeletonBow} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.bogged_bow_shoot @a[r=16] ~ ~ ~ 1 1`);
  let attackCount = 0;
  const maxAttacks = 3;
  const delay = 2; // delay in ticks
  const shoot = () => {
    if (attackCount < maxAttacks) {
      util.shootRangedProjectile(4, "sbs_mw:skeleton_bow_projectile", player,1.25);
      attackCount++;
      system.runTimeout(shoot, delay); // Schedule the next attack
    }
  };
  shoot(); // Start the first attack
  util.damageItem(player, reduceDurability.skeletonBow);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function pillagerCrossbow(event) {
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.pillagerCrossbow} default 0 "v.is_first_person"`);
  util.shootMultiRangedProjectile(4, 3, 45, "sbs_mw:pillager_crossbow_projectile", player,1.25);
  player.runCommand(`playsound sbs_mw.bogged_bow_shoot @a[r=16] ~ ~ ~ 1 1`);
  util.damageItem(player, reduceDurability.pillagerCrossbow);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function strayBow(event) {
  const abilityDelay = 2; // delay in ticks
  const player = event.source;
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 12`);
  player.runCommand(`playanimation @s ${abilityAnimation.strayBow} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.bogged_bow_shoot @a[r=16] ~ ~ ~ 1 1`);
  const ability = () => {
    util.shootRangedProjectile(4, "sbs_mw:stray_bow_projectile", player,1.25);
    util.damageItem(player, reduceDurability.strayBow);
  }
  system.runTimeout(ability, abilityDelay); // Schedule ability
}

////////////////////////////////////////////////////////////////////////////////////////////////////
