import { EquipmentSlot, ItemStack } from "@minecraft/server";

import * as util from "./util.js";

////////////////////////////////////////////////////////////////////////////////////////////////////

// list of durability reduction
const reduceDurability = {
  goatHelmetHit: 5,
  goatHelmetActivate: 20,
}

// animations
const abilityAnimation = {
  goatHelmet: "animation.sbs_mw.goat_helmet.player.tp_ability",
}

////////////////////////////////////////////////////////////////////////////////////////////////////

export function goatHelmet(event) {
  const player = event.source;
  const helmet_hand = event.itemStack;
  activateGoatAbility(player)
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimStep 10`);
  player.runCommand(`scoreboard players set @s sbs_mw_playerTpAnimTick 100`);
  player.runCommand(`playanimation @s ${abilityAnimation.goatHelmet} default 0 "v.is_first_person"`);
  player.runCommand(`playsound sbs_mw.goat_helmet_ability @a[r=10] ~ ~ ~ 1 1`);
  // set helmet to head
  const equip = player.getComponent('equippable');
  const helmet_head = new ItemStack("sbs_mw:goat_helmet_head", 1);
  helmet_head.lockMode = 'slot'
  // set cooldown
  const cd_comp = helmet_head.getComponent('cooldown');
  cd_comp.startCooldown(player);
  // copy durability
  const durability_hand = helmet_hand.getComponent('durability');
  // clear helmet from hand
  equip.setEquipment(EquipmentSlot.Head, helmet_head);
  equip.setEquipment(EquipmentSlot.Mainhand);
  player.setDynamicProperty("sbs_mw:goat_helmet_durability_damage", durability_hand.damage);
  // console.log(`helmet damage = ${player.getDynamicProperty("sbs_mw:goat_helmet_durability_damage")}`);
}

export function goatAbilityHandler(player) {
  const goating = player.getDynamicProperty("sbs_mw:player_goating") || 0;
  if (goating > 0) {
    const targets = util.getEntitiesFromRadius(player, 3);
    for (const target of targets) {
      target.runCommand(`particle sbs_mw:ravager_punch_hit ~ ~1 ~`);
      goatHelmetKnockback(player, target);
    }
    player.setDynamicProperty("sbs_mw:player_goating", goating - 1);
  }
  if (goating == 1) {
    // give helmet with cooldown
    const inv = player.getComponent('inventory');
    const helmet = new ItemStack("sbs_mw:goat_helmet", 1);
    // set cooldown
    const cd_comp = helmet.getComponent('cooldown');
    cd_comp.startCooldown(player);
    // copy durability
    const equip = player.getComponent('equippable');
    const durability_hand = helmet.getComponent('durability');
    try {
      durability_hand.damage = player.getDynamicProperty("sbs_mw:goat_helmet_durability_damage") + reduceDurability.goatHelmetActivate;
    } catch {
      durability_hand.damage = durability_hand.maxDurability
    }
    // set item if durability above 0
    if (durability_hand.damage < durability_hand.maxDurability) inv.container.addItem(helmet);
    // clear helmet from head
    equip.setEquipment(EquipmentSlot.Head, null);
  }
}

function goatHelmetKnockback(player, target) {
  try {
    // data
    const playerPosition = player.location;
    const targetPosition = target.location;
    const dx = targetPosition.x - playerPosition.x;
    const dz = targetPosition.z - playerPosition.z;
    const dist = Math.sqrt(dx * dx + dz * dz); // Horizontal distance between player and target

    // calculate the direction vector from the player to the target
    const dirX = dx / dist;
    const dirZ = dz / dist;

    // knockback power (higher for closer entities)
    const maxStrength = 4.0;
    const minStrength = 3.0;
    const strengthFactor = maxStrength - (dist / 3) * (maxStrength - minStrength); // Linear interpolation

    // apply knockback
    target.applyKnockback(dirX, dirZ, strengthFactor * 1.5, strengthFactor * 1.5);
    if (!target.applyDamage(5, { cause: "entityAttack", damagingEntity: player })) target.runCommand(`kill @s`);
    player.runCommand(`playsound sbs_mw.goat_helmet_hit @a[r=16] ~ ~ ~ 1 1`);
  } catch (err) {
    // console.log(`KNOCKBACK ERROR :: ${err}`);
  }
}

function activateGoatAbility(player) {
  const sec = 5
  try {
    player.setDynamicProperty("sbs_mw:player_goating", sec * 20);
  } catch {
    console.log(`INFO: Could not activate goat ability player: ${player.name}`);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
